/*
 * ds3231.c
 *
 *  Created on: Nov 13, 2022
 *      Author: hussamaldean
 */


#include "ds3231.h"
#include "i2c.h"


#define DS3231_I2C_ADDR             0x68


// time keeping registers
#define DS3231_TIME_CAL_ADDR        0x00
#define DS3231_ALARM1_ADDR          0x07
#define DS3231_ALARM2_ADDR          0x0B
#define DS3231_CONTROL_ADDR         0x0E
#define DS3231_STATUS_ADDR          0x0F
#define DS3231_AGING_OFFSET_ADDR    0x10
#define DS3231_TEMPERATURE_ADDR     0x11

// control register bits
#define DS3231_CONTROL_A1IE     	0x1		/* Alarm 2 Interrupt Enable */
#define DS3231_CONTROL_A2IE     	0x2		/* Alarm 2 Interrupt Enable */
#define DS3231_CONTROL_INTCN    	0x4		/* Interrupt Control */
#define DS3231_CONTROL_RS1	    	0x8		/* square-wave rate select 2 */
#define DS3231_CONTROL_RS2    		0x10	/* square-wave rate select 2 */
#define DS3231_CONTROL_CONV    		0x20	/* Convert Temperature */
#define DS3231_CONTROL_BBSQW    	0x40	/* Battery-Backed Square-Wave Enable */
#define DS3231_CONTROL_EOSC	    	0x80	/* not Enable Oscillator, 0 equal on */


// status register bits
#define DS3231_STATUS_A1F      		0x01		/* Alarm 1 Flag */
#define DS3231_STATUS_A2F      		0x02		/* Alarm 2 Flag */
#define DS3231_STATUS_BUSY     		0x04		/* device is busy executing TCXO */
#define DS3231_STATUS_EN32KHZ  		0x08		/* Enable 32KHz Output  */
#define DS3231_STATUS_OSF      		0x80		/* Oscillator Stop Flag */


static uint8_t dectobcd(const uint8_t val)
{
    return ((val / 10 * 16) + (val % 10));
}

static uint8_t bcdtodec(const uint8_t val)
{
    return ((val / 16 * 10) + (val % 16));
}


void DS3231_set(ts t)
{
    uint8_t i, century;
    century = 0x80;
    t.year_s = t.year - 2000;
    uint8_t TimeDate[7] = { t.sec, t.min, t.hour, t.wday, t.mday, t.mon, t.year_s };

    for (i=0;i<=6;i++)
    	{
    	TimeDate[i] = dectobcd(TimeDate[i]);
    	if(i == 5){TimeDate[5] |= century;}
    	}

    i2c_WriteMulti(DS3231_I2C_ADDR,DS3231_TIME_CAL_ADDR,(char*)TimeDate,7);
}

void DS3231_get(ts *t)
{
	uint8_t TimeDate[7];        //second,minute,hour,dow,day,month,year
	uint8_t i;
	uint16_t year_full;

	i2c_ReadMulti(DS3231_I2C_ADDR,DS3231_TIME_CAL_ADDR,7,(char*)TimeDate);

	for (i = 0; i <= 6; i++) {
		if (i == 5) {
			TimeDate[5] = bcdtodec(TimeDate[i] & 0x1F);

		} else
			TimeDate[i] = bcdtodec(TimeDate[i]);
	}


	year_full = 2000 + TimeDate[6];

	t->sec = TimeDate[0];
	t->min = TimeDate[1];
	t->hour = TimeDate[2];
	t->mday = TimeDate[4];
	t->mon = TimeDate[5];
	t->year = year_full;
	t->wday = TimeDate[3];
	t->year_s = TimeDate[6];

}

void print_time(ts *t)
{
	printf("Current DS3231 Time: %d:%d:%d\r\n",t->hour,t->min,t->sec);
	printf("Current DS3231 Date: %d/%d/%d\r\n",t->year,t->mon,t->mday);

}
